interface Dosa
{
 public String makeDosa();
}

class PlainDosa implements Dosa
{
 
 @Override
 public String makeDosa()
 {
 return "Plain Dosa";
 }
}

abstract class DosaDecorator implements Dosa
{
 
 protected Dosa dosa;
 
 public DosaDecorator( Dosa dosa )
 {
 this.dosa = dosa;
 }
 
 public String makeDosa()
 {
 return dosa.makeDosa();
 }
}

class MasalaDosa extends DosaDecorator
{
 
 public MasalaDosa( Dosa dosa )
 {
 super(dosa);
 }
 
 public String makeDosa()
 {
 return dosa.makeDosa() + addMasala();
 }
 
 private String addMasala()
 {
 return ",Masala added";
 
 }
}

class OnionDosa extends DosaDecorator
{
 
 public OnionDosa( Dosa dosa )
 {
 super(dosa);
 }
 
 public String makeDosa()
 {
 return dosa.makeDosa() + addOnion();
 
 }
 
 private String addOnion()
 {
 return ",Onion added";
 }
}

class MysoreDosa extends DosaDecorator
{
 
 public MysoreDosa( Dosa dosa )
 {
 super(dosa);
 }
 
 public String makeDosa()
 {
 return dosa.makeDosa() + addMysoreMasala();
 
 }
 
 private String addMysoreMasala()
 {
 return ",Mysore Masala added";
 }
}


public class DecoratorDesignPattern {
	 public static void main( String args[] )
	 {
	 PlainDosa plainDosaObj = new PlainDosa();
	 String plainDosa = plainDosaObj.makeDosa();
	 System.out.println(plainDosa);
	 
	 String onionDosa = new OnionDosa(plainDosaObj).makeDosa();
	 System.out.println("\n'" + onionDosa + "' using OnionDosa");
	 
	 String masalaDosa = new MasalaDosa(plainDosaObj).makeDosa();
	 System.out.println("\n'" + masalaDosa + "' using MasalaDosa");
	 
	 String mysoreDosa = new MysoreDosa(plainDosaObj).makeDosa();
	 System.out.println("\n'" + mysoreDosa + "' using MysoreDosa");
	 
	 }
}
